      Program Einstein
      Implicit None

C     Compute The Free Energy Of A Fcc Crystal
C     Using Einstein Integration

      Include 'commons.inc'
      
      Logical Laccept
      Integer I,J,Ncycle,Ix,Iy,Iz,Iref,Nc,Nmove,Ncell
      Double Precision Upot,Uein,Upotnew,Ueinnew,Upotold,Ueinold,Xnew
     $     ,Ynew,Znew,Vol,Du,Acc1,Acc2,Av1,Av2,RandomNumber,Cell,Cell2
     $     ,Cmx,Cmy,Cmz,Cmxnew,Cmynew,Cmznew,Dx,Dy,Dz,Delta,Avr1,Beta,Temp

      Open(21,File="Input",Status="Unknown")
      Read(21,*)
      Read(21,*) Ncycle,Ncell,Vol,Temp,Alpha,Delta,Lambda
      Close(21)

      Npart = 4*(Ncell**3)

      Write(6,*) 'Ncycle      : ',Ncycle
      Write(6,*) 'Ncell       : ',Ncell
      Write(6,*) 'Npart       : ',Npart
      Write(6,*) 'Volume      : ',Vol
      Write(6,*) 'Temperature : ',Temp
      Write(6,*) 'Alpha       : ',Alpha
      Write(6,*) 'Delta       : ',Delta
      Write(6,*) 'Lambda      : ',Lambda
  
      Beta = 1.0d0/Temp

      If(Npart.Gt.Maxpart.Or.Npart.Le.0) Stop "Error Npart !!!"
      If(Alpha.Lt.0.0d0) Stop "Error Alpha !!!"
      If(Lambda.Lt.0.0d0.Or.Lambda.Gt.1.0d0) Stop "Error Lambda !!!"

      Avr1 = 2.5d0**6
      Avr1 = 1.0d0/Avr1
      Ecut = 4.0d0*Avr1*(Avr1-1.0d0)

      Acc1   = 0.0d0
      Acc2   = 0.0d0
      Av1    = 0.0d0
      Av2    = 0.0d0
      Box    = Vol**(1.0d0/3.0d0)
      Ibox   = 1.0d0/Box
      Rcutsq = (2.5d0**2)
      
      Write(6,*) 'Box         : ',Box
      Write(6,*) 'Rho         : ',Dble(Npart)/Vol
      Write(6,*)

      If(Box.Lt.2.5d0) Then
         Write(6,*) 'Box is Too Small !!'
         Stop
      Endif

      If(Delta.Lt.0.0d0.Or.Delta.Gt.1.25d0) Then
         Write(6,*) 'Error Maximum Displacement !!!'
         Stop
      Endif

C     Generate Fcc Lattice

      Nc    = Nint((0.25d0*Dble(Npart))**(1.0d0/3.0d0))
      Cell  = Box/Dble(Nc)
      Cell2 = 0.5d0*Cell

      Rx(1) = 0.0d0
      Ry(1) = 0.0d0
      Rz(1) = 0.0d0

      Rx(2) = Cell2
      Ry(2) = Cell2
      Rz(2) = 0.0d0

      Rx(3) = 0.0d0
      Ry(3) = Cell2
      Rz(3) = Cell2

      Rx(4) = Cell2
      Ry(4) = 0.0d0
      Rz(4) = Cell2

      I = 0

      Do Iz=1,Nc
         Do Iy=1,Nc
            Do Ix=1,Nc
               Do Iref=1,4
                  Rx(Iref+I) = Rx(Iref) + Cell*Dble(Ix-1)
                  Ry(Iref+I) = Ry(Iref) + Cell*Dble(Iy-1)
                  Rz(Iref+I) = Rz(Iref) + Cell*Dble(Iz-1)
               Enddo
               I=I+4
            Enddo
         Enddo
      Enddo

C     Set Center Of Mass To Zero
C     Store Einstein Equilibrium Positions

      Cmx = 0.0d0
      Cmy = 0.0d0
      Cmz = 0.0d0

      Do I=1,Npart
         Cmx = Cmx + Rx(I)
         Cmy = Cmy + Ry(I)
         Cmz = Cmz + Rz(I)
      Enddo

      Cmx = Cmx/Dble(Npart)
      Cmy = Cmy/Dble(Npart)
      Cmz = Cmz/Dble(Npart)

      Do I=1,Npart
         Rx(I) = Rx(I) - Cmx
         Ry(I) = Ry(I) - Cmy
         Rz(I) = Rz(I) - Cmz

         Rxlat(I) = Rx(I)
         Rylat(I) = Ry(I)
         Rzlat(I) = Rz(I)
      Enddo

      Cmx = 0.0d0
      Cmy = 0.0d0
      Cmz = 0.0d0

      Call Etot(Upot,Uein,Cmx,Cmy,Cmz)

      Write(6,*) 'Upot        : ',Upot
      Write(6,*) 'Uein        : ',Uein
      Write(6,*)
      Write(6,*)

C     Loop Over All Cycles

      Open(21,File="Traject.xyz",Status="Unknown")

      Nmove = Ncycle*Npart

      Do I=1,Nmove
         J = 1 + Int(Dble(Npart)*RandomNumber())

C     Displace Randomly Selected Particle
         
         Dx = Delta*(Randomnumber()-0.5d0)
         Dy = Delta*(Randomnumber()-0.5d0)
         Dz = Delta*(Randomnumber()-0.5d0)

         Xnew = Rx(J) + Dx
         Ynew = Ry(J) + Dy
         Znew = Rz(J) + Dz

C     Update The New Center Of Mass

         Cmxnew = Cmx + Dx/Dble(Npart)
         Cmynew = Cmy + Dy/Dble(Npart)
         Cmznew = Cmz + Dz/Dble(Npart)
 
         Call Epart(Upotold,Ueinold,Rx(J),Ry(J),Rz(J),Cmx,Cmy,Cmz,J)
         Call Epart(Upotnew,Ueinnew,Xnew,Ynew,Znew,Cmxnew,Cmynew,Cmznew,J)

         Acc2 = Acc2 + 1.0d0

         Du = (Upotnew-Upotold)*(1.0d0-Lambda) + (Ueinnew-Ueinold)*Lambda
         Du = -Beta*Du

         If(Du.Gt.0.0d0) Then
            Laccept = .True.
         Elseif(Du.Lt.-75.0d0) Then
            Laccept = .False.
         Else
            If(RandomNumber().Lt.Dexp(Du)) Then
               Laccept = .True.
            Else
               Laccept = .False.
            Endif
         Endif

         If(Laccept) Then
            Upot   = Upot   + Upotnew - Upotold
            Uein   = Uein   + Ueinnew - Ueinold
            
            Cmx = Cmxnew
            Cmy = Cmynew
            Cmz = Cmznew

            Rx(J) = Xnew
            Ry(J) = Ynew
            Rz(J) = Znew

            Acc1 = Acc1 + 1.0d0
         Endif

         If(I.Gt.Nmove/3) Then
            Av1 = Av1 + Uein - Upot
            Av2 = Av2 + 1.0d0
         Endif

         If(Mod(I,Nmove/100).Eq.0) Then
            Write(21,*) Npart
            Write(21,*)
            Do J=1,Npart
               Write(21,'(A,3f15.5)') 'Ar  ',4.0d0*Rx(J),4.0d0*Ry(J),4.0d0*Rz(J)
            Enddo
         Endif
      Enddo
      
      Close(21)

      Call Etot(Upotnew,Ueinnew,Cmx,Cmy,Cmz)

      Write(6,*)
      Write(6,*) 'Fraction Accepted     : ',Acc1/Max(0.5d0,Acc2)
      Write(6,*) 'Uein                  : ',Ueinnew
      Write(6,*) 'Upot                  : ',Upotnew
      Write(6,*) 'Diff Uein             : ',Dabs(Ueinnew-Uein)
      Write(6,*) 'Diff Upot             : ',Dabs(Upotnew-Upot)
      Write(6,*)
      Write(6,*)
      Write(6,*) '<dU/dLambda>          : ',Av1/Max(0.5d0,Av2)
      Write(6,*)

C      Open(21,File="Coordnew",Status="Unknown")
C      Write(21,*) Box
C      Write(21,*) Npart
C      Do I=1,Npart
C
C         Rx(I) = Rx(I) - Cmx
C         Ry(I) = Ry(I) - Cmy
C         Rz(I) = Rz(I) - Cmz
C
C         If(Rx(I).Lt.0.0d0) Then
C            Rx(I) = Rx(I) + Box
C         Elseif(Rx(I).Gt.Box) Then
C            Rx(I) = Rx(I) - Box
C         Endif
C
C         If(Ry(I).Lt.0.0d0) Then
C            Ry(I) = Ry(I) + Box
C         Elseif(Ry(I).Gt.Box) Then
C            Ry(I) = Ry(I) - Box
C         Endif
C
C         If(Rz(I).Lt.0.0d0) Then
C            Rz(I) = Rz(I) + Box
C         Elseif(Rz(I).Gt.Box) Then
C            Rz(I) = Rz(I) - Box
C         Endif
C
C         Write(21,*) Rx(I),Ry(I),Rz(I)
C      Enddo
C      Close(21)

      Stop
      End
