      Program Mixture
      Implicit None

      Logical Lreaction
      Integer Maxpart,Maxbin

      Parameter (Maxpart = 1000)
      Parameter (Maxbin  = 10000)

      Integer I,J,Icycle,Ncycle,Icycle2,Icycle3,Id(Maxpart),Npart,Na,Nb
     $     ,Nid(2),Ido,Idn

      Double Precision Eground(2),Estart,Epart(Maxpart),Delta
     $     ,Randomnumber,Ui,Uj,Rm,Db,Av1,Av2,Av3,Move1,Move2,Move3,Move4
     $     ,Del,Idelta,Dist1(Maxbin,2),Dist2(2),Avk1,Avk2,Avk3,Dk1(Maxbin),Dk2

      Parameter (Delta = 0.01d0)
      Parameter (Idelta = 1.0d0/Delta)

C     Read Input

      Open(21,File="Input",Status="Unknown")
      Read(21,*)
      Read(21,*) Ncycle
      Read(21,*)
      Read(21,*) Lreaction,Npart,Na,Nb
      Read(21,*)
      Read(21,*) Eground(1),Eground(2),Estart
      Close(21)

C     Check Number Of Particles

      If(Na+Nb.Ne.Npart.Or.Npart.Gt.Maxpart.Or.Npart.Lt.3) 
     &     Stop "Error Npart (Too Many Or Too Few) !!!"
     
      Write(6,*) 'Ncycle             : ',Ncycle
      Write(6,*) 'Lreaction          : ',Lreaction
      Write(6,*) 'Npart              : ',Npart
      Write(6,*) 'Na                 : ',Na
      Write(6,*) 'Nb                 : ',Nb
      Write(6,*) 'Eground_A          : ',Eground(1)
      Write(6,*) 'Eground_B          : ',Eground(2)
      Write(6,*) 'Estart             : ',Estart
     
C     Maximum Displacement

      Del = Estart/Dble(2*Npart)

C     Reset Distributions

      Do I=1,Maxbin
         Dk1(I)     = 0.0d0
         Dist1(I,1) = 0.0d0
         Dist1(I,2) = 0.0d0
      Enddo

      Dk2      = 0.0d0
      Dist2(1) = 0.0d0
      Dist2(2) = 0.0d0
      Avk1     = 0.0d0
      Avk2     = 0.0d0
      Avk3     = 0.0d0

C     Identity Of The Particles

      Do I=1,Na
         Id(I) = 1
      Enddo

      Do I=Na+1,Npart
         Id(I) = 2
      Enddo

C     Distribute Initial Energy

      Do I=1,Npart
         Epart(I) = Estart/Dble(Npart)
      Enddo

      Nid(1) = Na
      Nid(2) = Nb

      Av1 = 0.0d0
      Av2 = 0.0d0
      Av3 = 0.0d0

      Move1 = 0.0d0
      Move2 = 0.0d0
      Move3 = 0.0d0
      Move4 = 0.0d0

      Write(6,*) 'Initial E          : ',
     &     Dble(Na)*Eground(1) + Dble(Nb)*Eground(2) + Estart
      Write(6,*)

      Open(22,File="Kt",Status="Unknown")

C     Start Of The Simulation

      Do Icycle=1,Ncycle
         Do Icycle2=1,1000
            Do Icycle3=1,Npart

               Rm = Randomnumber()

               If(Rm.Lt.0.5d0.And.Lreaction) Then

C     Identity Change

                  I = 1 + Int(Dble(Npart)*Randomnumber())

                  Move2 = Move2 + 1.0d0

                  Ido = Id(I)

                  If(Ido.Eq.1) Then
                     Idn = 2
                  Else
                     Idn = 1
                  Endif

                  Ui = Eground(Ido) + Epart(I) - Eground(Idn)

                  If(Ui.Gt.0.0d0) Then
                     Move1 = Move1 + 1.0d0

                     Epart(I) = Ui
                     Id(I)    = Idn

                     Nid(Ido) = Nid(Ido) - 1
                     Nid(Idn) = Nid(Idn) + 1
                  Endif
               Else

C     Energy Exchange

 1                Continue
                  I = 1 + Int(Dble(Npart)*Randomnumber())
                  J = 1 + Int(Dble(Npart)*Randomnumber())
                  If(I.Eq.J) Goto 1
               
                  Db = 2.0d0*Del*(Randomnumber()-0.5d0)

                  Move4 = Move4 + 1.0d0

                  Ui = Epart(I) + Db
                  Uj = Epart(J) - Db

                  If(Min(Ui,Uj).Gt.0.0d0) Then
                     Move3 = Move3 + 1.0d0

                     Epart(I) = Ui
                     Epart(J) = Uj
                  Endif
               Endif

C     Sample Averages

               If(Icycle.Gt.Ncycle/2) Then
                  Av1 = Av1 + Dble(Nid(1))
                  Av2 = Av2 + Dble(Nid(2))
                  Av3 = Av3 + 1.0d0

                  If(Nid(1).Ne.0) Then
                     Avk1 = Avk1 + Dble(Nid(2))/Dble(Nid(1))
                     Avk2 = Avk2 + (Dble(Nid(2))/Dble(Nid(1)))**2
                     Avk3 = Avk3 + 1.0d0

                     I = 1 + Int(Idelta*(Dble(Nid(2))/Dble(Nid(1))))
                     If(I.Le.Maxbin) Dk1(I) = Dk1(I) + 1.0d0
                     Dk2 = Dk2 + 1.0d0
                  Endif

                  J = 1 + Int(Dble(Npart)*Randomnumber())
                  I = 1 + Int(Epart(J)*Idelta)

                  If(I.Le.Maxbin) Dist1(I,Id(J)) = Dist1(I,Id(J)) + 1.0d0
               
                  Dist2(Id(J)) = Dist2(Id(J)) + 1.0d0
               Endif
            Enddo
         Enddo

         If(Nid(1).Ne.0) Write(22,*) Icycle,Dble(Nid(2))/Dble(Nid(1))
      Enddo
      
      Close(22)

C     Check Energy And Particle Balance

      Na     = Nid(1)
      Nb     = Nid(2)
      Nid(1) = 0
      Nid(2) = 0

      Ui = 0.0d0

      Do I=1,Npart
         If(Epart(I).Lt.0.0d0) Stop "Epart Negative !!!!"
         Ui         = Ui         + Epart(I) + Eground(Id(I))
         Nid(Id(I)) = Nid(Id(i)) + 1
      Enddo

      Write(6,*) 'Final E            : ',Ui
      Write(6,*) 'Final Na           : ',Na,Nid(1)
      Write(6,*) 'Final Nb           : ',Nb,Nid(2)
      Write(6,*)
      Write(6,*) '<Na>               : ',Av1/Av3
      Write(6,*) '<Nb>               : ',Av2/Av3
      Write(6,*) '<Kt>               : ',(Av2/Av3)/(Av1/Av3)
      Write(6,*) 'Spread <Kt>        : ',Dsqrt(Dabs((Avk2/Avk3) - ((Avk1/Avk3)**2)))
      Write(6,*)
      Write(6,*) 'Fraction Idchange  : ',Move1/Max(0.5d0,Move2)
      Write(6,*) 'Fraction Exchange  : ',Move3/Max(0.5d0,Move4)

C     Write Results

      Open(21,File="DistributionA",Status="Unknown")
      Do I=1,Maxbin
         If(Dist1(I,1).Gt.0.5d0) Write(21,*) 
     &        Eground(1)+(Dble(I)-0.5d0)*Delta,Dist1(I,1)/Dist2(1)
      Enddo
      Close(21)

      Open(21,File="DistributionB",Status="Unknown")
      Do I=1,Maxbin
         If(Dist1(I,2).Gt.0.5d0) Write(21,*) 
     &        Eground(2)+(Dble(I)-0.5d0)*Delta,Dist1(I,2)/Dist2(2)
      Enddo
      Close(21)

      Open(21,File="DistriKt",Status="Unknown")
      Do I=1,Maxbin
         If(Dk1(I).Gt.0.5d0) Write(21,*) (Dble(I)-0.5d0)*Delta,Dk1(I)/Dk2
      Enddo
      Close(21)

      Stop
      End
