      Program Ising
      Implicit None

C     2d Ising Model

      Integer LatticeSize,Iup(4),Jup(4),Magnet,Energy,I,J,K,Inew,Jnew
     $     ,NumberOfCycles,Maxlattice,Maxmag,Icycle,Icycle2,Kk,X1,X2,Y1
     $     ,Y2,Factor

      Parameter (Maxlattice  = 100)
      Parameter (Maxmag      = Maxlattice**2)

      Integer Lattice(Maxlattice,Maxlattice)

      Double Precision RandomNumber,Beta,EnergySum,EnergyCount
     $     ,Distribution(-Maxmag:Maxmag),Mag2Sum,Mag2Count,Energy2Sum
     $     ,Energy2Count,Weight,Dummy

C     Read Info

      Open(21,File="Input",Status="Unknown")
      Read(21,*)
      Read(21,*) NumberOfCycles,Beta,LatticeSize
      Close(21)

      Write(6,*) 'Ncycle                     : ',NumberOfCycles
      Write(6,*) 'Beta                       : ',Beta
      Write(6,*) 'Temperature                : ',1.0d0/Beta
      Write(6,*) 'Lattice Size               : ',LatticeSize
      Write(6,*)

      If(NumberOfCycles.Lt.100) Stop "Not Enough Cycles !!!"
      If(Latticesize.Gt.Maxlattice) Stop "Lattice Too Large !!!"

      EnergySum = 0.0d0
      EnergyCount = 0.0d0
      Energy2Sum = 0.0d0
      Energy2Count = 0.0d0
      Mag2Sum = 0.0d0
      Mag2Count = 0.0d0
             
      Do I=-Maxmag,Maxmag
         Distribution(I) = 0.0d0
      Enddo

C     Initialize Neighbours

      Iup(1) =  1
      Jup(1) =  0
      
      Iup(2) = -1
      Jup(2) =  0

      Iup(3) =  0
      Jup(3) =  1
      
      Iup(4) =  0
      Jup(4) = -1

      Do Icycle=1,NumberOfCycles
         Do Icycle2=1,100

C     Random Lattice

            Do I=1,LatticeSize
               Do J=1,LatticeSize
                  If(RandomNumber().Lt.0.5d0) Then
                     Lattice(I,J) =  1
                  Else
                     Lattice(I,J) = -1
                  Endif
               Enddo
            Enddo

            Energy = 0
            Magnet = 0

            Do I=1,LatticeSize
               Do J=1,LatticeSize

                  Magnet = Magnet + Lattice(I,J)

                  Do K=1,4
                     Inew   = I + Iup(K)
                     Jnew   = J + Jup(K)

                     If(Inew.Lt.1) Then
                        Inew = Inew + LatticeSize
                     Elseif(Inew.Gt.LatticeSize) Then
                        Inew = Inew - LatticeSize
                     Endif

                     If(Jnew.Lt.1) Then
                        Jnew = Jnew + LatticeSize
                     Elseif(Jnew.Gt.LatticeSize) Then
                        Jnew = Jnew - LatticeSize
                     Endif

                     Energy = Energy - 
     &                    Lattice(I,J)*Lattice(Inew,Jnew)
                  Enddo
               Enddo
            Enddo
         
            Energy = Energy/2

            Weight = Dexp(-Beta*Dble(Energy))

            EnergySum = EnergySum + Dble(Energy)*Weight
            EnergyCount = EnergyCount + Weight
         
            Energy2Sum = Energy2Sum + Dble(Energy**2)*Weight
            Energy2Count = Energy2Count + Weight

            Mag2Sum = Mag2Sum + Dble(Magnet**2)*Weight
            Mag2Count = Mag2Count + Weight

            Distribution(Magnet) = 
     &           Distribution(Magnet) + Weight
         Enddo

         If(Mod(Icycle,Numberofcycles/99).Eq.0) Then
            J = Icycle/(Numberofcycles/99) - 1

            If(J.Le.99) Then
               Open(21,File="Fig"//Char(48+J/10)//Char(48+Mod(J,10)),Status="Unknown")
               Write(21,'(A)') '#FIG 3.2'
               Write(21,'(A)') 'Landscape'
               Write(21,'(A)') 'Center'
               Write(21,'(A)') 'Inches'
               Write(21,'(A)') 'Letter'
               Write(21,'(A)') '100.00'
               Write(21,'(A)') 'Single'
               Write(21,'(A)') '-2'
               Write(21,'(A)') '1200 2'

               Do I=1,Latticesize
                  Do J=1,Latticesize

                     Factor = 200

                     If(Lattice(I,J).Gt.0.0d0) Then
                        Kk = 4
                     Else
                        Kk = 1
                     Endif

                     Write(21,'(A,2i5,A)') '2 2 0 1',Kk,Kk,' 50 -1 20 0.000 0 0 -1 0 0 5'

                     X1 = 1000 + (I-1)*Factor
                     X2 = X1 + Factor - 1
                     Y1 = 1000 + (J-1)*Factor
                     Y2 = Y1 + Factor - 1

                     Write(21,'(10i10)') X1,Y1,X2,Y1,X2,Y2,X1,Y2,X1,Y1
                  Enddo
               Enddo
               Close(21)
            Endif
         Endif
      Enddo

      Write(6,*) 'Average Energy             : ',
     &     EnergySum/EnergyCount

      Write(6,*) 'Average M*M                : ',
     &     Mag2Sum/Mag2Count

      Write(6,*) 'Heat Capacity              : ',
     &     Beta*Beta*((Energy2Sum/Energy2Count) - (EnergySum/EnergyCount)**2)

      Dummy = -2.0d0*Latticesize*Latticesize

      Write(6,*) 'Low T Estimate             : ', 
     &     (Dummy +  Latticesize*Latticesize*(Dummy + 8.0d0)*Exp(-8.0d0*Beta))/
     &     (1.0d0 + Latticesize*Latticesize*Exp(-8.0d0*Beta))

      Write(6,*)
      
      Open(21,File="magnetic.dat")

      Do I=-Maxmag,Maxmag
         If(Distribution(I).Gt.0.5d0) Then
            Write(21,*) I,Distribution(I)
         Endif
      Enddo   

      Close(21)
      
      Stop
      End
