      Program Mc
      Implicit None

C     Monte Carlo In Nvt Ensemble

      Include 'commons.inc'

      Logical Linit
      Integer Ncycle,Ninit,Nmove,I,J,Icycle,Icycle2
      Double Precision Temp,Deltax,V,E,Avd1,Avd2

      Avd1 = 0.0d0
      Avd2 = 0.0d0

      Open(21,File="Input",Status="Unknown")
      Read(21,*)
      Read(21,*) Ncycle,Ninit,Npart,Linit,Temp
      Read(21,*)
      Read(21,*) Deltax,Box
      Close(21)

      Beta   = 1.0d0/Temp
      Rcutsq = (2.5d0)**2
      Ecut   = Rcutsq**3
      Ecut   = 1.0d0/Ecut
      Ecut   = 4.0d0*Ecut*(Ecut-1.0d0)

      Write(6,*) 'Ncycle               : ',Ncycle
      Write(6,*) 'Ninit                : ',Ninit
      Write(6,*) 'Linit                : ',Linit
      Write(6,*) 'Temp                 : ',Temp
      Write(6,*) 'Deltax               : ',Deltax
          
      If(Temp.Le.0.0d0)   Stop "Error Temperature !!!"
      If(Deltax.Lt.0.0d0) Stop "Error Deltax !!"
      If(Ncycle.Lt.100)   Stop "Minimal 100 Cycles !!!"

      If(Linit) Then
         Write(6,*)
         Write(6,*) 'Generate Initial Coordinates'
         Write(6,*)

         If(Npart.Lt.0.Or.Npart.Gt.Maxpart) Stop "Error Npart !!!"
         If(Box.Lt.5.0d0) Stop "Boxes Too Small !!!"

         Call Init
      Else
         Write(6,*)
         Write(6,*) 'Read Coordinates From Disk'
         Write(6,*)

         Open(21,File="Coordold",Status="Unknown")
         Read(21,*) Box
         Read(21,*) Npart

         If(Npart.Lt.0.Or.Npart.Gt.Maxpart) Stop "Error Npart !!!"
         If(Box.Lt.5.0d0) Stop "Boxes Too Small !!!"

         Do I=1,Npart
            Read(21,*) Rx(I),Ry(I),Rz(I)
         Enddo
      Endif
      
      Write(6,*) 'Box                  : ',Box
      Write(6,*) 'Npart                : ',Npart
      Write(6,*) '<Rho>                : ',Dble(Npart)/(Box**3)
     
      If(Deltax.Gt.0.25d0*Box) Stop "Deltax Too Large !!!"

      Etotal = 0.0d0
      Vtotal = 0.0d0
      
      Call Etot(V,E)

      Vtotal = V
      Etotal = E
      
      Write(6,*)
      Write(6,*) 'Initial Energy Box   : ',Etotal
      Write(6,*) 'Initial Virial Box   : ',Vtotal
      Write(6,*)

C     Start Of The Simulation

      Write(6,*)
      Write(6,*)
      Write(6,*) 'The Simulation Is Running.....'
      Write(6,*)
      Write(6,*)

      Call Sample_Radial(1)
      Call Sample(1)

      Open(22,File="Traject.xyz",Status="Unknown")
      Open(23,File="Results",Status="Unknown")

      Do Icycle=1,Ncycle

         Nmove = Max(20,Npart)

         Do Icycle2=1,Nmove
            Call Move(Avd1,Avd2,Deltax)

            If(Icycle.Gt.Ninit) Call Sample(2)
         Enddo

         Write(23,'(3e20.10)') 
     &        Dble(Icycle),
     &        Etotal/Max(0.5d0,Dble(Npart)),
     &        Dble(Npart)/(Beta*(Box**3)) + Vtotal/(3.0d0*(Box**3))

         If(Icycle.Gt.Ninit) Then
            Call Sample_Radial(2)
         Endif

         If(Mod(Icycle,Ncycle/100).Eq.0) Then
            Write(22,*) Npart
            Write(22,*)

            Do J=1,Npart
               Write(22,'(A,3f15.5)') 'Ar  ',4.0d0*Rx(J),4.0d0*Ry(J),4.0d0*Rz(J)
            Enddo
         Endif
      Enddo

      Close(22)
      Close(23)

      Call Sample_Radial(3)
      Call Sample(3)
     
      Write(6,*) 'Frac. Acc. Displ.    : ',Avd1/Max(0.5d0,Avd2)
     
      Open(21,File="Coordnew",Status="Unknown")
      Write(21,*) Box
      Write(21,*) Npart

      Do I=1,Npart
         Write(21,'(3e20.10)') Rx(I),Ry(I),Rz(I)
      Enddo
      Close(21)

C     Check Energy Calculation

      Call Etot(V,E)
       
      Write(6,*)
      Write(6,*) 'Energy               : ',E
      Write(6,*) 'Energy Sim.          : ',Etotal
      Write(6,*) 'Diff                 : ',Dabs(Etotal-E)
      Write(6,*) 'Virial               : ',V
      Write(6,*) 'Virial Sim.          : ',Vtotal
      Write(6,*) 'Diff                 : ',Dabs(Vtotal-V)
      Write(6,*)
      
      Stop
      End
