      Program Ising
      Implicit None

C     2d Ising Model 32 by 32

      Integer LatticeSize,Iup(4),Jup(4),Magnet,Energy,I,J,Inew,Jnew
     $     ,Lnew,Lold,Diff,Ilat,Jlat,K,L,NumberOfCycles,Ninit ,Maxmag
     $     ,Mnew,Mold,x1,x2,y1,y2,Factor,Kk

      Parameter (LatticeSize = 32)
      Parameter (Maxmag      = 1024)

      Logical Linit
      Integer Lattice(Latticesize,Latticesize)

      Double Precision RandomNumber,Beta,EnergySum,
     &     Dist2,Weight,Accepted,Attempts,Dist3,EnergyCount,
     &     W(0:Maxmag),Distribution(-Maxmag:Maxmag)

C     Read Info

      Open(21,File="Input",Status="Unknown")
      Read(21,*)
      Read(21,*) NumberOfCycles,Beta,Linit
      Close(21)

      Write(6,*) 'Ncycle                     : ',NumberOfCycles
      Write(6,*) 'Beta                       : ',Beta
      Write(6,*) 'Linit                      : ',Linit
    
      If(NumberOfCycles.Lt.100) Stop "Not Enough Cycles !!"

      Do I=0,Maxmag
         W(I) = 1.0d0
      Enddo
      
C     Umbrella Sampling

      If(.Not.Linit) Then

         Write(6,*)
         Write(6,*) 'Reading Weightfunction wold.dat'
         Write(6,*)

         Open(21,File="wold.dat",Status="Unknown")

         Do I=0,Maxmag,2
            Read(21,*) K,EnergySum
         
            If(Abs(K).Gt.Maxmag.Or.Mod(K,2).Ne.0) Then
               Write(6,*) 'I Out Of Range !!'
               Stop
            Endif
            
            W(K) = EnergySum
         Enddo
      
         Close(21)
      Else
         Write(6,*)
         Write(6,*) 'Weightfunction Is Zero'
         Write(6,*) 
      Endif

      Ninit = NumberOfCycles/3
      EnergySum = 0.0d0
      EnergyCount = 0.0d0
      Accepted = 0.0d0
      Attempts = 0.0d0
          
      Do I=-Maxmag,Maxmag
         Distribution(I) = 0.0d0
      Enddo

C     Initialize Lattice                  
C     Random Lattice                      

      Do I=1,LatticeSize
         Do J=1,LatticeSize
            If(RandomNumber().Lt.0.5d0) Then
               Lattice(I,J) =  1
            Else
               Lattice(I,J) = -1
            Endif
         Enddo
      Enddo

C     Initialize Neighbours

      Iup(1) =  1
      Jup(1) =  0
      
      Iup(2) = -1
      Jup(2) =  0

      Iup(3) =  0
      Jup(3) =  1
      
      Iup(4) =  0
      Jup(4) = -1

C     Calculate Initial Energy      
C     Magnet = Total Magnetisation

      Energy = 0
      Magnet = 0

      Do I=1,LatticeSize
         Do J=1,LatticeSize

            Magnet = Magnet + Lattice(I,J)

            Do K=1,4
               Inew   = I + Iup(K)
               Jnew   = J + Jup(K)

               If(Inew.Lt.1) Then
                  Inew = Inew + LatticeSize
               Elseif(Inew.Gt.LatticeSize) Then
                  Inew = Inew - LatticeSize
               Endif

               If(Jnew.Lt.1) Then
                  Jnew = Jnew + LatticeSize
               Elseif(Jnew.Gt.LatticeSize) Then
                  Jnew = Jnew - LatticeSize
               Endif

               Energy = Energy - 
     &              Lattice(I,J)*Lattice(Inew,Jnew)
            Enddo
         Enddo
      Enddo

      Energy = Energy/2

      Write(6,*)
      Write(6,*) 'Lattice Size               : ',LatticeSize
      Write(6,*) 'Beta                       : ',Beta
      Write(6,*) 'Number Of Cycles           : ',NumberOfCycles
      Write(6,*) 'Number Of Init             : ',Ninit
      Write(6,*)
      Write(6,*)
      Write(6,*) 'Initial Energy             : ',Energy
      Write(6,*) 'Initial Magnetization      : ',Magnet
      Write(6,*)
      
      If(Abs(Magnet).Gt.Maxmag) Stop

C     Loop Over All Cycles

      Do K=1,NumberOfCycles
         Do L=1,100*LatticeSize*LatticeSize
            
C     Flip A Single Spin     
C     Metropolis Algorithm  

            Ilat = 1 + Idint(RandomNumber()*Dble(LatticeSize))
            Jlat = 1 + Idint(RandomNumber()*Dble(LatticeSize))
            
            Mold  = Magnet
            Lold  = Lattice(Ilat,Jlat)
            Lnew  = -Lold
            Mnew  = Mold + Lnew - Lold
            Diff  = 0
            Attempts = Attempts + 1.0d0
            
C     Calculate The Energy Difference
C     Between New And Old 

            Do I=1,4
               Inew = Ilat + Iup(I)
               Jnew = Jlat + Jup(I)

               If(Inew.Lt.1) Then
                  Inew = Inew + LatticeSize
               Elseif(Inew.Gt.LatticeSize) Then
                  Inew = Inew - LatticeSize
               Endif

               If(Jnew.Lt.1) Then
                  Jnew = Jnew + LatticeSize
               Elseif(Jnew.Gt.LatticeSize) Then
                  Jnew = Jnew - LatticeSize
               Endif

               Diff = Diff + 
     &              Lattice(Inew,Jnew)*(Lold-Lnew)
            Enddo

C     Acceptance/Rejection Rule 
C     Use Weight Function W     

            If(RandomNumber().Lt.
     &           Dexp((-Beta*Dble(Diff)) + 
     &           W(Abs(Mnew)) - W(Abs(Mold)))) Then

C     Update The Lattice/Energy/Magnetisation

               Accepted = Accepted + 1.0d0

               Lattice(Ilat,Jlat) = Lnew
               Energy             = Energy + Diff
               Magnet             = Mnew

               If(Abs(Magnet).Gt.Maxmag) Then
                  Write(6,*) 'Increase The Value Of Maxmag !!'
                  Stop
               Endif
            Endif
                  
            If(K.Gt.Ninit) Then
               Weight = Dexp(-W(Abs(Magnet)))
               EnergySum = EnergySum + Weight*Dble(Energy)
               EnergyCount = EnergyCount + Weight
               
               Distribution(Magnet) = 
     &              Distribution(Magnet) + 1.0d0
            Endif
         Enddo

         If(Mod(K,Numberofcycles/99).Eq.0) Then
            J = K/(Numberofcycles/99) - 1

            If(J.Le.99) Then
               Open(21,File="Fig"//Char(48+J/10)//Char(48+Mod(J,10)),Status="Unknown")
               Write(21,'(A)') '#FIG 3.2'
               Write(21,'(A)') 'Landscape'
               Write(21,'(A)') 'Center'
               Write(21,'(A)') 'Inches'
               Write(21,'(A)') 'Letter'
               Write(21,'(A)') '100.00'
               Write(21,'(A)') 'Single'
               Write(21,'(A)') '-2'
               Write(21,'(A)') '1200 2'

               Do I=1,Latticesize
                  Do J=1,Latticesize

                     Factor = 200

                     If(Lattice(I,J).Gt.0.0d0) Then
                        Kk = 4
                     Else
                        Kk = 1
                     Endif

                     Write(21,'(A,2i5,A)') '2 2 0 1',Kk,Kk,' 50 -1 20 0.000 0 0 -1 0 0 5'

                     X1 = 1000 + (I-1)*Factor
                     X2 = X1 + Factor - 1
                     Y1 = 1000 + (J-1)*Factor
                     Y2 = Y1 + Factor - 1

                     Write(21,'(10i10)') X1,Y1,X2,Y1,X2,Y2,X1,Y2,X1,Y1
                  Enddo
               Enddo
               Close(21)
            Endif
         Endif
      Enddo

      Write(6,*)
      Write(6,*) 'Average Energy             : ',
     &     EnergySum/EnergyCount

      Write(6,*) 'Fraction Accepted Swaps    : ',
     &     Accepted/Attempts

      Write(6,*)
      Write(6,*) 'Final Energy        (Simu) : ',Energy
      Write(6,*) 'Final Magnetization (Simu) : ',Magnet

C     Calculate Distributions 
C     Magnetisation           

      Dist2 = 0.0d0
      Dist3 = 0.0d0
            
      Do I=-Maxmag,Maxmag,2
         Weight = Dexp(-W(Abs(I)))

         Dist2  = Dist2 + Distribution(I)
         Dist3  = Dist3 + Distribution(I)*Weight
      Enddo

      Open(21,File="magnetic.dat")

      Do I=-Maxmag,Maxmag,2
         If(Distribution(I).Gt.0.5d0) Then
            Weight = Dexp(-W(Abs(I)))
            
            Write(21,*) I,Distribution(I)/Dist2,
     &           Distribution(I)/Dist3*Weight
         Endif
      Enddo   

      Close(21)

C     Compute New Estimate Weight Distribution

      Do I=0,Maxmag,2
         W(I) = W(I) - 0.5d0*Dlog(Max(1.0d0,(Distribution(I)+Distribution(-I))))
      Enddo

      Dist2 = W(0)

      Do I=0,Maxmag,2
         Dist2 = Min(Dist2,W(I))
      Enddo

      Do I=0,Maxmag,2
         W(I) = W(I) - Dist2
      Enddo

      Open(21,File="wnew.dat")
      Do I=0,Maxmag,2
         Write(21,*) I,W(I)
      Enddo
      Close(21)
      
C     Calculate Final Energy 
C     Used To Check The Code 

      Energy = 0
      Magnet = 0

      Do I=1,LatticeSize
         Do J=1,LatticeSize

            Magnet = Magnet + Lattice(I,J)

            Do K=1,4
               Inew   = I + Iup(K)
               Jnew   = J + Jup(K)

               If(Inew.Lt.1) Then
                  Inew = Inew + LatticeSize
               Elseif(Inew.Gt.LatticeSize) Then
                  Inew = Inew - LatticeSize
               Endif

               If(Jnew.Lt.1) Then
                  Jnew = Jnew + LatticeSize
               Elseif(Jnew.Gt.LatticeSize) Then
                  Jnew = Jnew - LatticeSize
               Endif

               Energy = Energy - 
     &              Lattice(I,J)*Lattice(Inew,Jnew)
            Enddo
         Enddo
      Enddo

      Energy = Energy/2

      Write(6,*) 'Final Energy        (Calc) : ',Energy
      Write(6,*) 'Final Magnetization (Calc) : ',Magnet
      Write(6,*)     

      Stop
      End
